<?php

namespace App\Http\Controllers;

use Validator;
use App\User;
use Firebase\JWT\JWT;
use Illuminate\Http\Request;
use Firebase\JWT\ExpiredException;
use Illuminate\Support\Facades\Hash;
use Laravel\Lumen\Routing\Controller as BaseController;
use Carbon\Carbon;
use DB;
use App\Page;
use App\PageFunction;
use App\PageGroup;
use App\UserGroup;
use App\UserGroupPermission;
use App\SystemUser;
use App\Merchant;
use App\Customer;
use App\Point;


class AuthController extends BaseController 
{
    /**
     * The request instance.
     *
     * @var \Illuminate\Http\Request
     */
    private $request;

    /**
     * Create a new controller instance.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return void
     */
    public function __construct(Request $request) {
        $this->request = $request;
    }

    /**
     * Create a new token.
     * 
     * @param  \App\User   $user
     * @return string
     */
    protected function jwt(User $user) {
        $payload = [
            'iss' => "lumen-jwt", // Issuer of the token
            'sub' => $user->id, // Subject of the token
            'iat' => time(), // Time when JWT was issued. 
            'exp' => time() + 60*60 // Expiration time
        ];
        
        // As you can see we are passing `JWT_SECRET` as the second parameter that will 
        // be used to decode the token in the future.
        return JWT::encode($payload, env('JWT_SECRET'));
    } 

    protected function jwt2(SystemUser $user) {
        $payload = [
            'iss' => "lumen-jwt", // Issuer of the token
            'sub' => $user->system_user_id, // Subject of the token
            'iat' => time(), // Time when JWT was issued. 
            'exp' => time() + 60*60 // Expiration time
        ];
        
        // As you can see we are passing `JWT_SECRET` as the second parameter that will 
        // be used to decode the token in the future.
        return JWT::encode($payload, env('JWT_SECRET'));
    } 

    /**
     * Authenticate a user and return the token if the provided credentials are correct.
     * 
     * @param  \App\User   $user 
     * @return mixed
     */
    public function authenticate(User $user) {
        $this->validate($this->request, [
            'email'     => 'required|email',
            'password'  => 'required'
        ]);

        // Find the user by email
        $user = User::where('email', $this->request->input('email'))->first();

        if (!$user) {
            // You wil probably have some sort of helpers or whatever
            // to make sure that you have the same response format for
            // differents kind of responses. But let's return the 
            // below respose for now.

            return response()->json([
                'error' => 'Email does not exist.','status' => 'fail'
            ], 400);
        }else{

        // Verify the password and generate the token
        if (Hash::check($this->request->input('password'), $user->password)) {
            $permissions = $this->get_user_and_permission($user->user_group);
            return response()->json([
                'token' => $this->jwt($user),'status' => 'success', 'result' => $user,'permissions' => $permissions ], 200);
        }

        // Bad Request response
        return response()->json([
            'error' => 'Password is wrong.','status' => 'fail'
        ], 400);
    }
    }


    public function get_user_and_permission($user_group){
    $permissions = UserGroupPermission::join('page_functions', 'page_functions.page_function_id', '=','user_group_permissions.page_function' )
            ->join('pages', 'pages.page_id', '=','page_functions.page')
            ->join('page_groups', 'page_groups.page_group_id', '=','pages.page_group' )
            ->where([
                ['user_group_permissions.user_group',$user_group],
               
            ])
            ->select("*")->orderBy('page_groups.page_group_position', 'asc')->get();
          
       return $permissions;
}

public function SignUpCustomer(Request $request)
    { 
        $first_name = $request->input('first_name');
        $last_name = $request->input('last_name');
        $email = $request->input('email_address');
        $phone_number = $request->input('phone_number');
        $birth_date = $request->input('birth_date');
        $password = Hash::make($request->input('password'));
        // $country = $request->input('country');
        $country = 'Ghana';
        $verified = $request->input('verified');
        $check_existence = SystemUser::where('email',$email)->where('user_type','customer')->first();
        if($check_existence) {// check for existence before saveing
            
            return response()->json([
                'status' => 'fail', 'result' => "",'msg' => "Customer Already Exist"], 200);
        }else{// save and return data


        $dataModel = new Customer();
        $dataModel->first_name = $first_name;
        $dataModel->last_name = $last_name;
        $dataModel->email_address = $email;
        $dataModel->phone_number = $phone_number;
        $dataModel->birth_date = $birth_date;
        $dataModel->fullname = $first_name.' '.$last_name;
        // 
        // $dataModel->password = $password;
        $dataModel->updated_at = Carbon::now();
        $dataModel->created_at = Carbon::now();
        $dataModel->save();
         if($dataModel){
         
            $new_record = Customer::where('email_address',$email)->first();
            //save point data
            $newPointModel = new Point();
            $newPointModel->user_type = 'customer';
            $newPointModel->user_type_id = $new_record->customer_id;
            $newPointModel->save();
             // save system user data
            $newSystemUserModel = new SystemUser();
            $newSystemUserModel->user_type = 'customer';
            $newSystemUserModel->user_type_id = $new_record->customer_id;
            $newSystemUserModel->email = $email;
            $newSystemUserModel->password = $password;
            $newSystemUserModel->save();

            return response()->json([
                'status' => 'success', 'result' => $new_record,'message' => "Account Created Successfully "], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Account Not Created"], 200);
        }
        }
        
    }

    public function SignUpMerchant(Request $request)
    { 
        $merchant_name = $request->input('merchant_name');
        $business_address = $request->input('business_address');
        $business_email = $request->input('business_email');
        $bus_contact_number = $request->input('bus_contact_no');
        $business_type = $request->input('business_type');
        $password = Hash::make($request->input('password'));
        // $country = $request->input('Country');
        $country = 'Ghana';
        $verified = $request->input('Verified');
        $email_verified_code = $request->input('EmailVerificationCode');
        $check_existence = SystemUser::where('email',$business_email)->where('user_type','merchant')->first();
        if($check_existence) {// check for existence before saveing
            
            return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Merchant Already Exist"], 200);
        }else{// save and return data


        $dataModel = new Merchant();
        $dataModel->merchant_name = $merchant_name;
        $dataModel->business_email = $business_email;
        $dataModel->business_address = $business_address;
        $dataModel->bus_contact_no = $bus_contact_number;
        $dataModel->business_type = $business_type;
        $dataModel->country = $country;
        $dataModel->verified = $verified;

        // $dataModel->password = $password;
        $dataModel->updated_at = Carbon::now();
        $dataModel->created_at = Carbon::now();
        $dataModel->save();
         if($dataModel){
            $new_record = Merchant::where('business_email',$business_email)->first();
             // $new_record = Customer::where('email_address',$email)->first();
            //save point data
            $newPointModel = new Point();
            $newPointModel->user_type = 'merchant';
            $newPointModel->user_type_id = $new_record->merchant_id;
            $newPointModel->save();
             // save system user data
            $newSystemUserModel = new SystemUser();
            $newSystemUserModel->user_type = 'merchant';
            $newSystemUserModel->user_type_id = $new_record->merchant_id;
            $newSystemUserModel->email = $business_email;
            $newSystemUserModel->password = $password;
            $newSystemUserModel->save();
            return response()->json([
                'status' => 'success', 'result' => $new_record,'message' => "Account Created Successfully"], 200);
        }else{
             return response()->json([
                'status' => 'fail', 'result' => "",'message' => "Account Not Created"], 200);
        }
        }
        
    }

    public function signin(Request $request) {
        $user_type = $this->request->input('user_type');
        // $this->validate($this->request, [
        //     'username'     => 'required|email',
        //     'password'  => 'required'
        // ]);
         // dd($request);
          // return response()->json(['res',$request]}
        // Find the user by email
        $user = SystemUser::where('email', $this->request->input('email'))->where('user_type',$user_type)->first();

        if (!$user) {
            // You wil probably have some sort of helpers or whatever
            // to make sure that you have the same response format for
            // differents kind of responses. But let's return the 
            // below respose for now.

            return response()->json([
                // 'error' => 'Email does not exist.','status' => 'fail'
                'msg' => 'Sorry Email Does Not Exist.','status' => 'fail'
            ], 200);
        }else{
        // Verify the password and generate the token
        if (Hash::check($this->request->input('password'), $user->password)) {
            if($user_type == "merchant"){
                 $user_data = SystemUser::join('merchants', 'merchants.merchant_id', '=','system_users.user_type_id')
             ->where('system_users.email', $this->request->input('email'))->where('system_users.user_type', 'merchant')->get();
              
            }else{
             $user_data = SystemUser::join('customers', 'customers.customer_id', '=','system_users.user_type_id')
             ->where('system_users.email', $this->request->input('email'))->where('system_users.user_type', 'customer')->get();
              
             }
            return response()->json([
                'token' => $this->jwt2($user),'status' => 'success', 'user' => $user_data,'msg'=>'Successfully Signed In' ], 200);
            
        }

        // Bad Request response
        return response()->json([
            // 'error' => 'Password is wrong.','status' => 'fail'
            'msg' => 'Password is Incorrect.','status' => 'fail'
        ], 200);
    }
    }

}

